@@ -1,31 +1,26 @@
-#!/usr/bin/python
 #
-# Copyright (c) 2017-2019 NVIDIA CORPORATION. All rights reserved.
+# Copyright (c) 2017-2021 NVIDIA CORPORATION. All rights reserved.
 # This file is part of the WebDataset library.
 # See the LICENSE file for licensing terms (BSD-style).
 #
 
+"""Automatically decode webdataset samples."""
 
-"""Train PyTorch models directly from POSIX tar archive, locally
-or over HTTP connections.
-"""
-
-__all__ = "WebDataset tariterator default_handlers imagehandler".split()
-
+import io
+import json
+import os
 import pickle
 import re
-import os
+import tempfile
+from functools import partial
 
 import numpy as np
-import json
-import tempfile
-import io
-
 
 from .checks import checkmember, checknotnone
 
 
-check_present = int(os.environ.get("WDS_CHECK_DECODE", 0))
+"""Extensions passed on to the image decoder."""
+image_extensions = "jpg jpeg png ppm pgm pbm pnm".split()
 
 
 ################################################################
@@ -34,6 +29,10 @@ check_present = int(os.environ.get("WDS_CHECK_DECODE", 0))
 
 
 def torch_loads(data):
+    """Load data using torch.loads, importing torch only if needed.
+
+    :param data: data to be decoded
+    """
     import io
     import torch
 
@@ -42,7 +41,23 @@ def torch_loads(data):
 
 
 def basichandlers(key, data):
+    """Handle basic file decoding.
 
+    This function is usually part of the post= decoders.
+    This handles the following forms of decoding:
+
+    - txt -> unicode string
+    - cls cls2 class count index inx id -> int
+    - json jsn -> JSON decoding
+    - pyd pickle -> pickle decoding
+    - pth -> torch.loads
+    - ten tenbin -> fast tensor loading
+    - mp messagepack msg -> messagepack decoding
+    - npy -> Python NPY decoding
+
+    :param key: file name extension
+    :param data: binary data to be decoded
+    """
     extension = re.sub(r".*[.]", "", key)
 
     if extension in "txt text transcript":
@@ -79,6 +94,47 @@ def basichandlers(key, data):
         stream = io.BytesIO(data)
         return numpy.lib.format.read_array(stream)
 
+    if extension in "npz".split():
+        return np.load(io.BytesIO(data))
+
+
+################################################################
+# Generic extension handler.
+################################################################
+
+def call_extension_handler(key, data, f, extensions):
+    """Call the function f with the given data if the key matches the extensions.
+
+    :param key: actual key found in the sample
+    :param data: binary data
+    :param f: decoder function
+    :param extensions: list of matching extensions
+    """
+    extension = key.lower().split(".")
+    for target in extensions:
+        target = target.split(".")
+        if len(target) > len(extension):
+            continue
+        if extension[-len(target):] == target:
+            return f(data)
+    return None
+
+
+def handle_extension(extensions, f):
+    """Return a decoder function for the list of extensions.
+
+    Extensions can be a space separated list of extensions.
+    Extensions can contain dots, in which case the corresponding number
+    of extension components must be present in the key given to f.
+    Comparisons are case insensitive.
+
+    Examples:
+    handle_extension("jpg jpeg", my_decode_jpg)  # invoked for any file.jpg
+    handle_extension("seg.jpg", special_case_jpg)  # invoked only for file.seg.jpg
+    """
+    extensions = extensions.lower().split()
+    return partial(call_extension_handler, f=f, extensions=extensions)
+
 
 ################################################################
 # handle images
@@ -105,34 +161,6 @@ imagespecs = {
 }
 
 
-def handle_extension(extensions, f):
-    """Returns a decoder function for the list of extensions.
-
-    Extensions can be a space separated list of extensions.
-    Extensions can contain dots, in which case the corresponding number
-    of extension components must be present in the key given to f.
-    Comparisons are case insensitive.
-
-    Examples:
-
-    handle_extension("jpg jpeg", my_decode_jpg)  # invoked for any file.jpg
-    handle_extension("seg.jpg", special_case_jpg)  # invoked only for file.seg.jpg
-    """
-    extensions = extensions.lower().split()
-
-    def g(key, data):
-        extension = key.lower().split(".")
-        for target in extensions:
-            target = target.split(".")
-            if len(target) > len(extension):
-                continue
-            if extension[-len(target) :] == target:
-                return f(data)
-        return None
-
-    return g
-
-
 class ImageHandler:
     """Decode image data using the given `imagespec`.
 
@@ -160,14 +188,26 @@ class ImageHandler:
 
     """
 
-    def __init__(self, imagespec):
+    def __init__(self, imagespec, extensions=image_extensions):
+        """Create an image handler.
+
+        :param imagespec: short string indicating the type of decoding
+        :param extensions: list of extensions the image handler is invoked for
+        """
         checkmember(imagespec, list(imagespecs.keys()), "unknown image specification")
         self.imagespec = imagespec.lower()
+        self.extensions = extensions
 
     def __call__(self, key, data):
+        """Perform image decoding.
+
+        :param key: file name extension
+        :param data: binary data
+        """
         import PIL.Image
+
         extension = re.sub(r".*[.]", "", key)
-        if extension.lower() not in "jpg jpeg png ppm pgm pbm pnm".split():
+        if extension.lower() not in self.extensions:
             return None
         imagespec = self.imagespec
         atype, etype, mode = imagespecs[imagespec]
@@ -198,8 +238,15 @@ class ImageHandler:
         return None
 
 
-def imagehandler(imagespec):
-    return ImageHandler(imagespec)
+def imagehandler(imagespec, extensions=image_extensions):
+    """Create an image handler.
+
+    This is just a lower case alias for ImageHander.
+
+    :param imagespec: textual image spec
+    :param extensions: list of extensions the handler should be applied for
+    """
+    return ImageHandler(imagespec, extensions)
 
 
 ################################################################
@@ -208,6 +255,11 @@ def imagehandler(imagespec):
 
 
 def torch_video(key, data):
+    """Decode video using the torchvideo library.
+
+    :param key: file name extension
+    :param data: data to be decoded
+    """
     extension = re.sub(r".*[.]", "", key)
     if extension not in "mp4 ogv mjpeg avi mov h264 mpg webm wmv".split():
         return None
@@ -218,7 +270,7 @@ def torch_video(key, data):
         fname = os.path.join(dirname, f"file.{extension}")
         with open(fname, "wb") as stream:
             stream.write(data)
-        return torchvision.io.read_video(fname)
+        return torchvision.io.read_video(fname, pts_unit="sec")
 
 
 ################################################################
@@ -227,6 +279,11 @@ def torch_video(key, data):
 
 
 def torch_audio(key, data):
+    """Decode audio using the torchaudio library.
+
+    :param key: file name extension
+    :param data: data to be decoded
+    """
     extension = re.sub(r".*[.]", "", key)
     if extension not in ["flac", "mp3", "sox", "wav", "m4a", "ogg", "wma"]:
         return None
@@ -257,10 +314,22 @@ class Continue:
     """
 
     def __init__(self, key, data):
+        """__init__.
+
+        :param key:
+        :param data:
+        """
         self.key, self.data = key, data
 
 
 def gzfilter(key, data):
+    """Decode .gz files.
+
+    This decodes compressed files and the continues decoding.
+
+    :param key: file name extension
+    :param data: binary data
+    """
     import gzip
 
     if not key.endswith(".gz"):
@@ -270,7 +339,7 @@ def gzfilter(key, data):
 
 
 ################################################################
-# a sample decoder
+# decode entire training amples
 ################################################################
 
 
@@ -285,7 +354,17 @@ class Decoder:
     handlers until some handler returns something other than None.
     """
 
-    def __init__(self, handlers, pre=None, post=None):
+    def __init__(self, handlers, pre=None, post=None, only=None):
+        """Create a Decoder.
+
+        :param handlers: main list of handlers
+        :param pre: handlers called before the main list (.gz handler by default)
+        :param post: handlers called after the main list (default handlers by default)
+        :param only: a list of extensions; when give, only ignores files with those extensions
+        """
+        if isinstance(only, str):
+            only = only.split()
+        self.only = only if only is None else set(only)
         if pre is None:
             pre = default_pre_handlers
         if post is None:
@@ -296,6 +375,11 @@ class Decoder:
         self.handlers = pre + handlers + post
 
     def decode1(self, key, data):
+        """Decode a single field of a sample.
+
+        :param key: file name extension
+        :param data: binary data
+        """
         key = "." + key
         for f in self.handlers:
             result = f(key, data)
@@ -307,6 +391,10 @@ class Decoder:
         return data
 
     def decode(self, sample):
+        """Decode an entire sample.
+
+        :param sample: the sample, a dictionary of key value pairs
+        """
         result = {}
         assert isinstance(sample, dict), sample
         for k, v in list(sample.items()):
@@ -315,10 +403,18 @@ class Decoder:
                     v = v.decode("utf-8")
                 result[k] = v
                 continue
+            if self.only is not None and k not in self.only:
+                result[k] = v
+                continue
             checknotnone(v)
+            assert isinstance(v, bytes)
             result[k] = self.decode1(k, v)
         return result
 
     def __call__(self, sample):
+        """Decode an entire sample.
+
+        :param sample: the sample
+        """
         assert isinstance(sample, dict), (len(sample), sample)
         return self.decode(sample)
